/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// AtariDisk.cpp: implementation of the CDiskImage class.
//
//////////////////////////////////////////////////////////////////////

/*
#ifdef WIN32
#include "stdafx.h"
#else if __GNUC__
#include "gnuDefines.h"
#endif
*/

#include "diskImage.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDiskImage::CDiskImage()
{
	path[0]=NULL;
	sectorSize=0;;
	sectorCount=0;
	paragraphs=0;
	fp=NULL;
	readOnly=false;
	happyFlag=false;
	memset(&atrHeader,0,sizeof(atrHeader));
	memset(&deviceConfig,0,sizeof(deviceConfig));
	memset(&deviceStatus,0,sizeof(deviceStatus));
}

CDiskImage::~CDiskImage()
{
	closeImage();
}


bool CDiskImage::createImage(LPSTR imagepath,WORD imageSectorCount, WORD imageSectorSize)
{
	LPCSTR src = "CDiskImage::createImage";

	DWORD attrs = GetFileAttributes(imagepath); 
    if (attrs & FILE_ATTRIBUTE_READONLY) 
    { 
		SetFileAttributes(imagepath, 
		    attrs & ~FILE_ATTRIBUTE_READONLY); 
    } 

	fp = fopen(imagepath,"wb");
	DWORD b;
	if(fp)
	{
		memset(&atrHeader,0,sizeof(atrHeader));
		atrHeader.atrNickAtari = ATR_SIGNATURE;
		atrHeader.sectorSize=imageSectorSize;

		//see Nick Kennedy's readme.txt for SIO2PC
		paragraphs = (imageSectorCount-3)*imageSectorSize/16 + 3*SIO_SD_SECTOR_SIZE/16;
		atrHeader.paragraphCountLo = LOWORD(paragraphs);
		atrHeader.paragraphCountHi = LOBYTE(HIWORD(paragraphs));

		b=fwrite(&atrHeader,sizeof(atrHeader),1,fp);
		if (b!=1)
		{
			innerShell_error(GetLastError(),src,"fwrite.1",NULL);
			closeImage();
			return false;
		}

		BYTE zdata[16]={0,0,0,0,
			            0,0,0,0,
					    0,0,0,0,
					    0,0,0,0};
		for(WORD i=0;i<paragraphs;i++)
		{
			b=fwrite(zdata,sizeof(zdata),1,fp);
			if (b!=1)
			{
				innerShell_error(GetLastError(),src,"fwrite.2",NULL);
				closeImage();
				return false;
			}
		}

		fclose(fp);
	}
	else
	{
		innerShell_error(GetLastError(),src,"fopen",NULL);
		return false;
	}

	return true;
}

bool CDiskImage::load(LPSTR imagepath, bool isReadOnly)
{
	LPCSTR src = "CDiskImage::loadImage";
	DWORD b;

	closeImage();

	wsprintf(path,"%s",imagepath);
	readOnly=isReadOnly;

	if(this->readOnly)
		fp = fopen(path,"rb");
	else
	{
		fp = fopen(path,"r+b");
		if (!fp && GetLastError()==5) //access denied
		{
			this->readOnly=true;
			fp = fopen(path,"rb");
		}
	}
	if(fp)
	{
		b=fread(&atrHeader,sizeof(atrHeader),1,fp);
		if (b!=1)
		{
			innerShell_error(GetLastError(),src,"fread",NULL);
			closeImage();
			return false;
		}

		if (atrHeader.atrNickAtari!=ATR_SIGNATURE)
		{
			innerShell_error(100,src,NULL,"invalid atr header identification bytes in %s",path);
			closeImage();
			return false;
		}

		sectorSize=atrHeader.sectorSize;

		//see Nick Kennedy's readme.txt for SIO2PC
		paragraphs=atrHeader.paragraphCountLo+atrHeader.paragraphCountHi*65536;
		sectorCount=LOWORD((16*paragraphs-3*SIO_SD_SECTOR_SIZE)/sectorSize+3);
	}
	else
	{
		innerShell_error(GetLastError(),src,"fopen",NULL);
		return false;
	}


	return fp!=NULL;
}

void CDiskImage::closeImage()
{
	if (fp)
	{
		fclose(fp);
		fp=NULL;
	}
	path[0]=NULL;
}

bool CDiskImage::isReadOnly()
{
	return readOnly;
}

WORD CDiskImage::getSectorSize()
{
	return sectorSize;
}

WORD CDiskImage::getSectorSize(WORD sector)
{
	WORD size;
	size=sectorSize;
	if (sector<=3) size=SIO_SD_SECTOR_SIZE;
	return size;
}

WORD CDiskImage::getSectorCount()
{
	return sectorCount;
}

bool CDiskImage::isHappy()
{
	return happyFlag;
}

void CDiskImage::setHappy(bool newState)
{
	happyFlag=newState;
}

DEVICE_CONFIG_BLOCK* CDiskImage::getDeviceConfig()
{
	deviceConfig.bytesPerSectorLo = LOBYTE(sectorSize); //discrepency between ATR header and DCB
	deviceConfig.deviceActivated=1;
	deviceConfig.sectorCountHi=HIBYTE(sectorCount);
	deviceConfig.sectorCountLo=LOBYTE(sectorCount);
	deviceConfig.stepRate=1;
	deviceConfig.trackCount=1;
	deviceConfig.checkSum=g_sio->checkSum((BYTE*)&deviceConfig,sizeof(deviceConfig)-1);
	return &deviceConfig;
}

bool CDiskImage::setDeviceConfig(DEVICE_CONFIG_BLOCK* dc)
{
	//do nothing for now
	return true;
}

DEVICE_STATUS_BLOCK* CDiskImage::getDeviceStatus()
{
	deviceStatus.motorOn=1;
	deviceStatus.timeout=1;
	deviceStatus.enhancedDensity=0;
	deviceStatus.doubleDensity=0;
	deviceStatus.writeProtect=readOnly?1:0;

	if (sectorSize==SIO_SD_SECTOR_SIZE && sectorCount>SIO_SSSD_SECTOR_NUM)
		deviceStatus.enhancedDensity=1;
	else if (sectorSize==SIO_DD_SECTOR_SIZE)
		deviceStatus.doubleDensity=1;

	deviceStatus.checksum=g_sio->checkSum((BYTE*)&deviceStatus,sizeof(deviceStatus)-1);
	return &deviceStatus;
}

WORD CDiskImage::writeSector(BYTE* sb,WORD sector)
{
	LPCSTR src="CDiskImage::writeSector";
	WORD size=0;
	DWORD off;
	DWORD i;

	if (sector <= sectorCount)
	{
		size=getSectorSize(sector);
		off = seekSector(sector);
		if (fseek(fp,off,SEEK_SET))
			innerShell_error(GetLastError(),src,"fseek",NULL);
		i=fwrite(sb,1,size,fp);
		if (i!=size)
			innerShell_error(GetLastError(),src,"fwrite",NULL);
	}

	return size;
}

WORD CDiskImage::readSector(BYTE* sb, WORD sector)
{
	WORD size;
	DWORD off;
	WORD i;
	LPCSTR src="CDiskImage::readSector";

	size = getSectorSize(sector);

	off = seekSector(sector);

	if (fseek(fp,off,SEEK_SET))
		innerShell_error(GetLastError(),src,"fseek",NULL);

	i=fread(sb,1,size,fp);
	if (i!=size)
		innerShell_error(GetLastError(),src,"fread",NULL);

	return size;
}

DWORD CDiskImage::seekSector(WORD sector)
{
	DWORD off=sizeof(ATR_HEADER);
	if (sector<4)
		off+=(sector-1)*SIO_SD_SECTOR_SIZE;
	else
		off+=3*SIO_SD_SECTOR_SIZE+(sector-4)*sectorSize;
	return off;
}

LPCSTR CDiskImage::description()
{
	wsprintf(desc,"%s [%d %d-byte sectors]",
					isReadOnly()?" read-only":"",
					getSectorCount(),getSectorSize());
	return desc;
}
