/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// EventQueue.cpp: implementation of the CEventQueue class.
//
//////////////////////////////////////////////////////////////////////

/*
#ifdef WIN32
#include "stdafx.h"
#else if __GNUC__
#include "gnuDefines.h"
#endif
*/

#include "EventQueue.h"
#define MSG_SIZE 500

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CEventQueue::CEventQueue()
{
	q = new queue<string*>;
	queueMutex = CreateMutex(NULL, false, NULL);
}

CEventQueue::~CEventQueue()
{
	CloseHandle(queueMutex);
	delete q;
}

DWORD CEventQueue::size()
{
	DWORD s;
	WaitForSingleObject(queueMutex, INFINITE);
	s=q->size();
	ReleaseMutex(queueMutex);
	return s;
}

string* CEventQueue::pop()
{
	string* s;
	WaitForSingleObject(queueMutex, INFINITE);
	s=q->front();
	q->pop();
	ReleaseMutex(queueMutex);
	return s;
}

void CEventQueue::push(string* s)
{
	WaitForSingleObject(queueMutex, INFINITE);
	q->push(s);
	ReleaseMutex(queueMutex);
}

void CEventQueue::pushDebug(LPCSTR msg)
{
	string* s;
	s = new string("D");
	s->append(msg);
	push(s);
}

void CEventQueue::pushStatus(LPCSTR msg)
{
	string* s;
	s = new string("S");
	s->append(msg);
	push(s);
}

void CEventQueue::pushError(DWORD number, LPCSTR src, LPCSTR func, LPCSTR msg)
{
	LPVOID pmsg=NULL;
	char bufsrc[MSG_SIZE];
	char bufend[MSG_SIZE];
	LPSTR t;

	if (number && !msg)
	{
#ifdef WIN32
		FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER |
			FORMAT_MESSAGE_FROM_SYSTEM |
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			number,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR)&pmsg,
			0,
			NULL
		);
		t=strchr((LPCSTR)pmsg,0x0d);
		if(t) *t=NULL;
		t=strchr((LPCSTR)pmsg,0x0a);
		if(t) *t=NULL;
#else if __GNUC__
		pmsg = "fatal error";
#endif
	}

	bufend[0]=NULL;
	bufsrc[0]=NULL;
	if(func && src)
		wsprintf(bufsrc,"(%s.%s)",src,func);
	else if (func || src)
		wsprintf(bufsrc,"(%s)",src?src:func);
	wsprintf(bufend,"%s %s",msg?msg:(LPCSTR)pmsg,bufsrc);

	BYTE* b;
	string* s;
	s = new string("E1234");
	s->append(bufend);
	b=(BYTE*)&number;

	for(int i=1;i<=4;i++)
		(*s)[i]=b[i-1];

	if (pmsg)
		LocalFree(pmsg);

	push(s);
}

void CEventQueue::pushCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex)
{
	BYTE* t;
	string* s;
	int i;
	s = new string("C123456789AB");

	s->append(command);

	(*s)[1]=b1;
	(*s)[2]=b2;
	(*s)[3]=b3;
	(*s)[4]=b4;
	(*s)[5]=b5;

	(*s)[6]=disk;
	(*s)[7]=speedIndex;

	t=(BYTE*)&sector;
	for(i=8;i<=0x9;i++)
		(*s)[i]=t[i-8];

	push(s);
}

void CEventQueue::pushDebugCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex)
{
	BYTE* t;
	string* s;
	int i;
	s = new string("U123456789AB");

	s->append(command);

	(*s)[1]=b1;
	(*s)[2]=b2;
	(*s)[3]=b3;
	(*s)[4]=b4;
	(*s)[5]=b5;

	(*s)[6]=disk;
	(*s)[7]=speedIndex;

	t=(BYTE*)&sector;
	for(i=8;i<=0x9;i++)
		(*s)[i]=t[i-8];

	push(s);
}

void CEventQueue::popEvent()
{
	string* s=pop();
	string sub;
	char evtType = (*s)[0];
	LPCSTR msg=NULL;
	DWORD number=0;
	BYTE* t=NULL;
	int i;

	switch(evtType)
	{
	case'E':
		t = (BYTE*)&number;
		for(i=1;i<=4;i++)
			t[i-1]=(*s)[i];
		sub=s->substr(5);
		innerShell_displayError(number,sub.c_str());
	break;
	case'S':
		sub=s->substr(1);
		innerShell_displayStatus(sub.c_str());
	break;
	case'D':
		if (g_state->isDebugSet()) break;
		sub=s->substr(1);
		innerShell_displayDebug(sub.c_str());
	break;
	case'C':
		t = (BYTE*)&number;
		sub=s->substr(12);
		for(i=8;i<=0x9;i++)
			t[i-8]=(*s)[i];
		innerShell_displayCommand((*s)[1],(*s)[2],(*s)[3],(*s)[4],(*s)[5],(*s)[6],LOWORD(number),sub.c_str(),(*s)[7]);
	break;
	case'U':
		t = (BYTE*)&number;
		sub=s->substr(12);
		for(i=8;i<=0x9;i++)
			t[i-8]=(*s)[i];
		innerShell_displayCommand((*s)[1],(*s)[2],(*s)[3],(*s)[4],(*s)[5],(*s)[6],LOWORD(number),sub.c_str(),(*s)[7]);
	break;
	}
 }

