/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version. This program is 
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// innerShell.cpp: implementation of the innerShell routines.
//
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
#include "windefines.h"
#else if __GNUC__
#include "gnuDefines.h"
#endif

#include <conio.h>
#include "innerShell.h"


CSIOProtocol* g_sio=NULL;
CStateStore* g_state=NULL;
CEventQueue* g_evq=NULL;
Shell* g_sh=NULL;
bool g_shownVersion=false;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


void innerShell_init(Shell* sh)
{
	if(!g_sio)
	{
		g_evq = new CEventQueue();
		g_state = new CStateStore();
		g_sio = new CSIOProtocol();
	}
	g_sh = sh;
}

void innerShell_destroy()
{
	g_sh = NULL;
	/*
	delete g_sio;
	delete g_evq;
	delete g_state;
	*/
}

bool innerShell_createImage(LPSTR imagepath,WORD imageSectorCount,WORD imageSectorSize)
{
	CDiskImage* di;
	di = new CDiskImage();
	if (di->createImage(imagepath,imageSectorCount,imageSectorSize))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool innerShell_mount(LPSTR path,BYTE diskid,bool readOnly)
{
	bool b;
	b=g_sio->mount(path,diskid,readOnly);
	return b;
}

bool innerShell_release(BYTE diskid)
{
	bool b;
	b=g_sio->release(diskid);
	return b;
}

void innerShell_emptyEventQueue()
{
	g_state->emptyQueue();
}

bool innerShell_setSerialPort(BYTE portNumber)
{
	bool b;
	bool restart=false;

	if (!g_state->isStopped()) 
	{	
		g_state->stop(true);
		restart = true;
	}
	b=g_sio->sioPort->setSerialPort(portNumber);
	if (b && restart) g_state->start();
	return b;
}

void innerShell_swapDisks(BYTE diskid1,BYTE diskid2)
{
	g_sio->swapDisks(diskid1,diskid2);
}

LPCSTR innerShell_getDiskPath(BYTE diskid)
{
	if(diskid>0 && diskid<=SIO_MAX_DISKS && g_sio->disk[diskid])
		return g_sio->disk[diskid]->path;
	else
		return "";
}

bool innerShell_getDiskReadOnly(BYTE diskid)
{
	if(diskid>0 && diskid<=SIO_MAX_DISKS && g_sio->disk[diskid])
		return g_sio->disk[diskid]->isReadOnly();
	else
		return false;
}

WORD innerShell_getDiskSectorSize(BYTE diskid)
{
	if(diskid>0 && diskid<=SIO_MAX_DISKS && g_sio->disk[diskid])
		return g_sio->disk[diskid]->getSectorSize();
	else
		return 0;
}

WORD innerShell_getDiskSectorCount(BYTE diskid)
{
	if(diskid>0 && diskid<=SIO_MAX_DISKS && g_sio->disk[diskid])
		return g_sio->disk[diskid]->getSectorCount();
	else
		return 0;
}

void innerShell_stop()
{
	g_state->stop();
}

bool innerShell_isStopped()
{
	return g_state->isStopped();
}

bool innerShell_isErrorSet()
{
	return g_state->isErrorSet();
}

bool innerShell_isStarted()
{
	return g_state->isStarted();
}

void innerShell_start()
{
	g_state->start();
}

void innerShell_displayStatus(LPCSTR msg)
{
	if(g_sh) g_sh->Status(msg);
}

void innerShell_displayDebug(LPCSTR msg)
{
	if(g_sh) g_sh->Debug(msg);
}

void innerShell_displayError(DWORD number, LPCSTR msg)
{
	if(g_sh) g_sh->Error(number,msg);
}

void innerShell_displayCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex)
{
	if(g_sh) g_sh->Command(b1,b2,b3,b4,b5,disk,sector,command,speedIndex);
	//printf("\n%02x %02x %02x %02x %02x",b1,b2,b3,b4,b5);
	//if (disk<=SIO_MAX_DISKS && disk>=1) printf(" [D%d: %s sector %d]",disk,command,sector);
}

void innerShell_error(DWORD number, LPCSTR src, LPCSTR func, LPCSTR fmt, ...)
{
	char chunk[MSG_SIZE];
	LPSTR s=NULL;

	va_list va;
	va_start(va, fmt);
	if (fmt)
	{
		wvsprintf(chunk, fmt, va);
		s=chunk;
	}
	va_end(va);
	g_evq->pushError(number,src,func,s);
	//g_state->setError(true);
	
	innerShell_emptyEventQueue();
	//throw number;
}

void innerShell_criticalSioError(DWORD number, LPCSTR src, LPCSTR func, LPCSTR fmt, ...)
{
	char chunk[MSG_SIZE];
	LPSTR s=NULL;

	va_list va;
	va_start(va, fmt);
	if (fmt)
	{
		wvsprintf(chunk, fmt, va);
		s=chunk;
	}
	va_end(va);
	g_evq->pushError(number,src,func,s);
	g_evq->pushError(number,src,func,"sio emulation has been halted due to critical error(s)");
	g_state->setError(true);
	
	innerShell_emptyEventQueue();
	//throw number;
}

void innerShell_status(LPCSTR fmt, ...)
{
	char chunk[MSG_SIZE];
	va_list va;
	va_start(va, fmt);
	wvsprintf(chunk, fmt, va);
	va_end(va);
	g_evq->pushStatus(chunk);
}

void innerShell_debug(LPCSTR fmt, ...)
{
	char chunk[MSG_SIZE];
	va_list va;
	va_start(va, fmt);
	wvsprintf(chunk, fmt, va);
	va_end(va);
	g_evq->pushDebug(chunk);
	innerShell_emptyEventQueue();
}

void innerShell_sioCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex)
{
	g_evq->pushCommand(b1,b2,b3,b4,b5,disk,sector,command,speedIndex);
}

void innerShell_debugSioCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex)
{
	g_evq->pushDebugCommand(b1,b2,b3,b4,b5,disk,sector,command,speedIndex);
}

BYTE innerShell_getMaxSpeed()
{
	return g_sio->sioPort->maxSpeed;
}

void innerShell_putMaxSpeed(BYTE newval)
{
	g_sio->sioPort->maxSpeed=newval;
}

bool innerShell_getShowDebugMessages()
{
	return g_state->isDebugSet();
}

void innerShell_putShowDebugMessages(bool newval)
{
	g_state->setDebug(newval);
}
