/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// AtariDisk.h: interface for the CDiskImage class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ATARIDISK_H__FC541AB0_177E_421A_8994_515F4EE8FB69__INCLUDED_)
#define AFX_ATARIDISK_H__FC541AB0_177E_421A_8994_515F4EE8FB69__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "sioConstants.h"
#include "innershell.h"

#pragma pack(1)

// this struct is from Nick Kennedy's readme.txt for SIO2PC
#define ATR_SIGNATURE 0x0296
typedef struct _atrHeader
{
	WORD atrNickAtari;     /* 0x0296 */
	WORD paragraphCountLo;
	WORD sectorSize;
	BYTE paragraphCountHi; /* v3.00 */
	DWORD crc32;           /* APE extension */
	BYTE unused[4];
	BYTE status;           /* APE extension : bit 0 = write protect, bit 1 = sealed/authenticated */
} ATR_HEADER;

// the format of this struct was posted to comp.sys.atari.8bit
// by Matthias Belitz, Feb 1, 1997
typedef struct _deviceConfigBlock
{
	BYTE trackCount;           //number of tracks [1 for harddisks)
	BYTE stepRate;             //step rate
	BYTE sectorCountHi;        //sectors/track high (total number for harddisks)
	BYTE sectorCountLo;        //sectors/track low
	BYTE headCount;            //number of heads decreased by one
	BYTE formatId;             //format (0=FM; 4=MFM)
	BYTE bytesPerSectorHi;
	BYTE bytesPerSectorLo;

	BYTE sectorStartWithZero:1;
	BYTE first3SectorsFullSize:1;
	BYTE unusedBits:4;
	BYTE deviceActivated:1;
	BYTE fourtyTrackOnEightyTrack:1;

	BYTE unused[3];
	BYTE checkSum;
} DEVICE_CONFIG_BLOCK;

typedef struct deviceStatusBlock
{
	BYTE commandFrameError:1;
	BYTE dataFrameError:1;
	BYTE writeError:1;
	BYTE writeProtect:1;
	BYTE motorOn:1;
	BYTE doubleDensity:1;
	BYTE unusedBit:1;
	BYTE enhancedDensity:1;

	BYTE controllerBusy:1;
	BYTE dataRequestOrIndex:1;
	BYTE dataLostOrTrack0:1;
	BYTE crcError:1;
	BYTE recordNotFound:1;
	BYTE recordType:1;
	BYTE writeProtected:1;
	BYTE notReady:1;

	BYTE timeout;
	BYTE unused;
	BYTE checksum;
} DEVICE_STATUS_BLOCK;
typedef struct _directoryEntry
{
	BYTE flags;
	WORD sectorsInFile;
	WORD startingSector;
	char filename[8];
	char extension[3];
} DIRECTORY_ENTRY;
typedef struct _vtoc
{
	WORD w0;
	WORD w1;
	WORD w2;
	BYTE unused[3];
	BYTE sectorMap[119];
} VTOC;

#pragma pack()

class CDiskImage
{
public:
	CDiskImage();
	virtual ~CDiskImage();

	virtual bool load(LPSTR imagepath, bool isReadOnly);
	virtual bool createImage(LPSTR imagepath,WORD imageSectorCount, WORD imageSectorSize);
	virtual bool isReadOnly();
	virtual WORD getSectorSize();
	virtual WORD getSectorSize(WORD sector);
	virtual WORD getSectorCount();
	virtual void closeImage();
	virtual WORD readSector(BYTE* sb, WORD sector);
	virtual WORD writeSector(BYTE* sb, WORD sector);
	virtual DEVICE_STATUS_BLOCK* getDeviceStatus();
	virtual bool setDeviceConfig(DEVICE_CONFIG_BLOCK* dc);
	virtual DEVICE_CONFIG_BLOCK* getDeviceConfig();
	virtual bool isHappy();
	virtual void setHappy(bool newState);
	virtual LPCSTR description();

	char path[2048];
	char desc[512];

protected:
	VTOC vtoc;
	DIRECTORY_ENTRY directorySector[8];
	WORD sectorSize;
	WORD sectorCount;
	DWORD paragraphs;
	bool readOnly;
	bool happyFlag;
	ATR_HEADER atrHeader;
	DEVICE_CONFIG_BLOCK deviceConfig;
	BYTE deviceStatusx[SIO_STATUS_SIZE+1];    //one byte extra for checksum
	DEVICE_STATUS_BLOCK deviceStatus;

private:
	DWORD seekSector(WORD sector);
	FILE *fp;

};

#endif // !defined(AFX_ATARIDISK_H__FC541AB0_177E_421A_8994_515F4EE8FB69__INCLUDED_)
